package com.fjhx.victoriatourist.controller.jd;

import cn.hutool.core.util.StrUtil;
import com.fjhx.victoriatourist.service.jd.JdApiService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.net.URLEncoder;


/**
 * <p>
 * 京东API 前端控制器
 * </p>
 *
 * @author
 * @since 2023-11-03
 */
@Slf4j
@Controller
@RequestMapping("/open/jd")
public class JdApiController {

    private static final String DOMAIN = "http://127.0.0.1:80";

    @Autowired
    private JdApiService jdApiService;

    /**
     * 京东登录回调
     */
    @RequestMapping("/callback")
    public void callback(HttpServletResponse response, String code, String state) throws IOException {
        log.info("京东回调code：{},state：{}", code, state);
        String msg = "";
        if (StrUtil.isBlank(code) || StrUtil.isBlank(state)) {
            msg = "参数错误";
        }

        //获取token并且入库
        try {
            jdApiService.getAccessTokenByCode(code);
        } catch (Exception e) {
            log.error("京东获取token异常", e);
            msg = "京东获取token异常";
        }
        response.sendRedirect(DOMAIN + "/callback.html?msg=" + URLEncoder.encode(msg));
    }

    @RequestMapping("/build2LoginUrl")
    @ResponseBody
    public String build2LoginUrl() {
        return jdApiService.build2LoginUrl();
    }

}
