package com.fjhx.victoriatourist.service.excess.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.extra.spring.SpringUtil;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.dynamic.datasource.toolkit.DynamicDataSourceContextHolder;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.common.constant.SourceConstant;
import com.fjhx.common.utils.Assert;
import com.fjhx.item.entity.product.po.ProductInfo;
import com.fjhx.item.service.product.ProductInfoService;
import com.fjhx.kd100.util.KD100Util;
import com.fjhx.victoriatourist.entity.excess.dto.ExcessGoodsDetailsDto;
import com.fjhx.victoriatourist.entity.excess.dto.ExcessGoodsDetailsSelectDto;
import com.fjhx.victoriatourist.entity.excess.po.ExcessGoodsDetails;
import com.fjhx.victoriatourist.entity.excess.po.ExcessGoodsRegister;
import com.fjhx.victoriatourist.entity.excess.vo.ExcessGoodsDetailsVo;
import com.fjhx.victoriatourist.entity.logistics.po.LogisticsInfos;
import com.fjhx.victoriatourist.mapper.excess.ExcessGoodsDetailsMapper;
import com.fjhx.victoriatourist.service.excess.ExcessGoodsDetailsService;
import com.fjhx.victoriatourist.service.excess.ExcessGoodsRegisterService;
import com.fjhx.victoriatourist.service.logistics.LogisticsInfosService;
import com.fjhx.wms.entity.stock.emums.StockWaitType;
import com.fjhx.wms.entity.stock.po.StockWait;
import com.fjhx.wms.entity.stock.po.StockWaitDetails;
import com.fjhx.wms.service.stock.StockWaitDetailsService;
import com.fjhx.wms.service.stock.StockWaitService;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.wrapper.IWrapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 多货登记明细 服务实现类
 * </p>
 *
 * @author
 * @since 2024-03-27
 */
@Service
public class ExcessGoodsDetailsServiceImpl extends ServiceImpl<ExcessGoodsDetailsMapper, ExcessGoodsDetails> implements ExcessGoodsDetailsService {

    @Autowired
    private ExcessGoodsRegisterService excessGoodsRegisterService;
    @Autowired
    private ProductInfoService productInfoService;
    @Autowired
    private StockWaitService stockWaitService;
    @Autowired
    private LogisticsInfosService logisticsInfosService;
    @Autowired
    private StockWaitDetailsService stockWaitDetailsService;

    private IWrapper<ExcessGoodsDetails> getWrapper(ExcessGoodsDetailsSelectDto dto) {
        IWrapper<ExcessGoodsDetails> wrapper = getWrapper();
        wrapper.eq("egd", ExcessGoodsDetails::getStockWaitId, dto.getStockWaitId());

        wrapper.orderByDesc("egd", ExcessGoodsDetails::getCreateTime);
        wrapper.orderByDesc("egd", ExcessGoodsDetails::getId);
        return wrapper;
    }

    @Override
    public List<ExcessGoodsDetailsVo> getList(ExcessGoodsDetailsSelectDto dto) {
        IWrapper<ExcessGoodsDetails> wrapper = getWrapper(dto);
        //过滤未提交的数据
        wrapper.eq("egd", ExcessGoodsDetails::getIsSubmit, 0);

        List<ExcessGoodsDetailsVo> list = this.baseMapper.getList(wrapper);
        setInfo(list);
        return list;
    }

    @Override
    public Page<ExcessGoodsDetailsVo> getPage(ExcessGoodsDetailsSelectDto dto) {
        IWrapper<ExcessGoodsDetails> wrapper = getWrapper(dto);

        //过滤已提交的数据
        wrapper.eq("egd", ExcessGoodsDetails::getIsSubmit, 1);

        //处理结果过滤
        wrapper.eq("egd", ExcessGoodsDetails::getProcessingMethod, dto.getProcessingMethod());

        //关键字检索
        String keyword = dto.getKeyword();
        if (ObjectUtil.isNotEmpty(keyword)) {
            List<Long> productIds = productInfoService.listObject(ProductInfo::getId,
                    q -> q.like(ProductInfo::getCustomCode, keyword).or().like(ProductInfo::getName, keyword));
            wrapper.and(q -> q
                    .like("sw.business_code", keyword)
                    .or().like("lis.logistics_company_code", keyword)
                    .or().like("lis.`code`", keyword)
                    .or().like("egd", ExcessGoodsDetails::getQuantity, keyword)
                    .or().like("egd", ExcessGoodsDetails::getBackLogisticsCode, keyword)
                    .or().in("egd", ExcessGoodsDetails::getProductId, productIds)
            );
        }

        Page<ExcessGoodsDetailsVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        List<ExcessGoodsDetailsVo> records = page.getRecords();
        if (ObjectUtil.isEmpty(records)) {
            return page;
        }
        setInfo(records);
        return page;
    }

    void setInfo(List<ExcessGoodsDetailsVo> list) {
        List<Long> ids = list.stream().map(ExcessGoodsDetails::getId).collect(Collectors.toList());
        if (ObjectUtil.isEmpty(ids)) {
            return;
        }
        //赋值清点列表
        Map<Long, List<ExcessGoodsRegister>> longListMap = excessGoodsRegisterService.mapKGroup(ExcessGoodsRegister::getExcessGoodsDetailsId, q -> q
                .in(ExcessGoodsRegister::getExcessGoodsDetailsId, ids)
        );

        for (ExcessGoodsDetailsVo excessGoodsDetailsVo : list) {
            excessGoodsDetailsVo.setExcessGoodsRegisterList(longListMap.getOrDefault(excessGoodsDetailsVo.getId(), new ArrayList<>()));
        }

        //赋值产品信息
        productInfoService.attributeAssign(list, ExcessGoodsDetails::getProductId, (item, productinfo) -> {
            item.setProductCode(productinfo.getCode());
            item.setProductCustomCode(productinfo.getCustomCode());
            item.setProductName(productinfo.getName());
            item.setProductSpec(productinfo.getSpec());
            item.setProductUnit(productinfo.getUnit());
        });
    }

    @Override
    public ExcessGoodsDetailsVo detail(Long id) {
        ExcessGoodsDetails ExcessGoodsDetails = this.getById(id);
        ExcessGoodsDetailsVo result = BeanUtil.toBean(ExcessGoodsDetails, ExcessGoodsDetailsVo.class);
        return result;
    }

    @Override
    public void add(ExcessGoodsDetailsDto dto) {
        Long stockWaitId = dto.getStockWaitId();
        Assert.notEmpty(stockWaitId, "待入库id不能为空");
        StockWait stockWait = stockWaitService.getById(stockWaitId);
        Assert.notEmpty(stockWait, "查询不到待入库信息");
        LogisticsInfos one = logisticsInfosService.getOne(q -> q.eq(LogisticsInfos::getBusinessId, stockWait.getDeliverGoodsId()));
        if (ObjectUtil.isNotEmpty(one)) {
            dto.setLogisticsInfosId(one.getId());
            dto.setPurchaseId(one.getPurchaseId());
        }
        this.save(dto);
    }

    @Override
    public void edit(ExcessGoodsDetailsDto excessGoodsDetailsDto) {
        this.updateById(excessGoodsDetailsDto);
    }

    @Override
    public void delete(Long id) {
        ExcessGoodsDetails byId = this.getById(id);
        if (ObjectUtil.notEqual(byId.getProcessingMethod(), 0)) {
            throw new ServiceException("该数据已被处理禁止删除！");
        }

        this.remove(q -> q
                .eq(ExcessGoodsDetails::getId, id)
                .eq(ExcessGoodsDetails::getProcessingMethod, 0)
        );
    }

    /**
     * 退货
     */
    @Override
    @DSTransactional
    public void backGoods(ExcessGoodsDetailsDto dto) {
        Assert.notEmpty(this.getById(dto.getId()), "查询不到多货信息！");
        dto.setProcessingMethod(10);//退货
        this.updateById(dto);

//        long logisticsInfosId = IdWorker.getId();
//        LogisticsInfos logisticsInfos = new LogisticsInfos();
//        logisticsInfos.setBusinessId(dto.getId());
//        logisticsInfos.setBusinessType(5);
//        logisticsInfos.setId(logisticsInfosId);
//        logisticsInfos.setIsKd100(1);
//        logisticsInfos.setExcessGoodsId(dto.getId());
//        logisticsInfos.setStockWaitId(excessGoodsDetails.getStockWaitId());
//        logisticsInfos.setPurchaseId(excessGoodsDetails.getPurchaseId());
//        //发起快递100订阅
//        String activeProfile = SpringUtil.getActiveProfile();
//        DynamicDataSourceContextHolder.push(SourceConstant.KD100);
//        Integer state = !"prod".equals(activeProfile) ? -1 : KD100Util.monitor(logisticsInfos.getLogisticsCompanyCode(), logisticsInfos.getCode(), logisticsInfosId, 1, LogisticsInfos.class).getState();
//        DynamicDataSourceContextHolder.poll();
//        logisticsInfos.setLogisticsStatus(state);
//        logisticsInfos.setIsKd100(1);
//        logisticsInfosService.save(logisticsInfos);
//        LogisticsDetails logisticsDetails = new LogisticsDetails();
//        logisticsDetails.setLogisticsInfoId(logisticsInfos.getId());
//        logisticsDetailsService.save(logisticsDetails);
    }

    /**
     * 入库
     */
    @DSTransactional
    @Override
    public void inStock(List<Long> excessGoodsDetailsIds) {
        ExcessGoodsDetails byId = this.getById(excessGoodsDetailsIds.get(0));
        StockWait byId1 = stockWaitService.getById(byId.getStockWaitId());

        //添加一份待出库数据
        StockWait stockWait = new StockWait();
        stockWait.setType(1);
        stockWait.setStatus(0);
        stockWait.setBusinessId(byId.getId());
        stockWait.setBusinessType(StockWaitType.EXCESS_GOODS_IN.getDetailType());
        stockWait.setPurchaseId(byId.getPurchaseId());
        stockWait.setBusinessCode(byId1.getBusinessCode());
        stockWaitService.save(stockWait);


        long logisticsInfosId = IdWorker.getId();

        LogisticsInfos logisticsInfos = new LogisticsInfos();

        LogisticsInfos byId2 = logisticsInfosService.getById(byId.getLogisticsInfosId());
        if (ObjectUtil.isNotEmpty(byId2)) {
            logisticsInfos.setLogisticsCompanyCode(byId2.getLogisticsCompanyCode());
            logisticsInfos.setCode(byId2.getCode());
        }

        logisticsInfos.setBusinessId(stockWait.getId());
        logisticsInfos.setBusinessType(1);
        logisticsInfos.setId(logisticsInfosId);
        logisticsInfos.setIsKd100(1);
        logisticsInfos.setStockWaitId(stockWait.getId());
        logisticsInfos.setPurchaseId(stockWait.getPurchaseId());
        //发起快递100订阅
        String activeProfile = SpringUtil.getActiveProfile();
        DynamicDataSourceContextHolder.push(SourceConstant.KD100);
        Integer state = !"prod".equals(activeProfile) ? -1 : KD100Util.monitor(logisticsInfos.getLogisticsCompanyCode(), logisticsInfos.getCode(), logisticsInfosId, 1, LogisticsInfos.class).getState();
        DynamicDataSourceContextHolder.poll();
        logisticsInfos.setLogisticsStatus(state);
        logisticsInfos.setIsKd100(1);
        logisticsInfosService.save(logisticsInfos);


        for (Long id : excessGoodsDetailsIds) {
            ExcessGoodsDetails baseExcessGoodsDetails = this.getById(id);
            Assert.notEmpty(baseExcessGoodsDetails, "查询不到多货信息！");
            baseExcessGoodsDetails.setProcessingMethod(20);//入库
            this.updateById(baseExcessGoodsDetails);


            StockWaitDetails stockWaitDetails = new StockWaitDetails();
            stockWaitDetails.setStockWaitId(stockWait.getId());
            stockWaitDetails.setProductId(baseExcessGoodsDetails.getProductId());
            stockWaitDetails.setQuantity(baseExcessGoodsDetails.getQuantity());

            stockWaitDetails.setBusinessDetailsId(baseExcessGoodsDetails.getId());
            stockWaitDetails.setExcessGoodsDetailsId(baseExcessGoodsDetails.getId());
            stockWaitDetailsService.save(stockWaitDetails);
        }


    }

}
