package com.fjhx.victoriatourist.service.jd.initializers;

import com.baomidou.dynamic.datasource.toolkit.DynamicDataSourceContextHolder;
import com.fjhx.common.constant.SourceConstant;
import com.fjhx.victoriatourist.entity.jd.vo.JdInfoVo;
import com.fjhx.victoriatourist.service.jd.JdApiService;
import com.fjhx.victoriatourist.service.jd.constants.RunParamConstant;
import com.ruoyi.common.utils.SecurityUtils;
import org.springframework.context.annotation.Profile;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * 京东初始化类
 *
 * @author hj
 * @date 2023/12/21 9:11
 */
@Profile("prod")
@Component
@EnableScheduling
public class JdInitializer {

    @Resource
    private JdApiService jdApiService;

    @Scheduled(fixedDelay = RunParamConstant.SLEEP_ONE_MINUTE)
    public void jdTask() {
        SecurityUtils.setTenantId(RunParamConstant.tenantId);
        DynamicDataSourceContextHolder.push(SourceConstant.VICTORIATOURIST);

        JdInfoVo jdInfoVo = jdApiService.validJdInfo();

        // 收集京东采购单
        jdApiService.startJdOrder(jdInfoVo);

        // 收集京东退货单
        jdApiService.startJdBackOrder(jdInfoVo);

        // 刷新京东订单状态与信息
        jdApiService.refreshJdState(jdInfoVo);

        // 初始化字典
        jdApiService.initTenantDict();

        // 补偿商品id为空
        jdApiService.compensatingProductNon();

        SecurityUtils.clearTenantId();
        DynamicDataSourceContextHolder.poll();
    }

}
