package com.fjhx.wms.controller.stock;

import com.baomidou.dynamic.datasource.annotation.DS;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fjhx.common.constant.SourceConstant;
import com.fjhx.item.entity.product.IssueImportExcelBo;
import com.fjhx.item.service.product.ProductInfoService;
import com.fjhx.wms.entity.stock.dto.StockDto;
import com.fjhx.wms.entity.stock.dto.StockSelectDto;
import com.fjhx.wms.entity.stock.po.Stock;
import com.fjhx.wms.entity.stock.po.StockAnalysis;
import com.fjhx.wms.entity.stock.vo.StockVo;
import com.fjhx.wms.service.stock.StockService;
import com.ruoyi.common.core.domain.BaseSelectDto;
import com.ruoyi.common.core.domain.entity.SysDept;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.util.List;


/**
 * <p>
 * 库存 前端控制器
 * </p>
 *
 * @author
 * @since 2023-03-20
 */
@DS(SourceConstant.WMS)
@RestController
@RequestMapping("/stock")
public class StockController {

    @Autowired
    private StockService stockService;
    @Autowired
    private ProductInfoService productInfoService;

    /**
     * 库存查询
     */
    @PostMapping("/page")
    public Page<StockVo> page(@RequestBody StockSelectDto dto) {
        return stockService.getPage(dto);
    }

    /**
     * 维多利亚 按仓库查询
     */
    @PostMapping("/pageByWarehouse")
    public Page<StockVo> pageByWarehouse(@RequestBody StockSelectDto dto) {
        return stockService.pageByWarehouse(dto);
    }

    /**
     * 维多利亚 按产品查询
     */
    @PostMapping("/pageByProduct")
    public Page<StockVo> pageByProduct(@RequestBody StockSelectDto dto) {
        return stockService.pageByProduct(dto);
    }

    /**
     * 维多利亚 按产品spu查询
     */
    @PostMapping("/pageByProductSpu")
    public Page<StockVo> pageByProductSpu(@RequestBody StockSelectDto dto) {
        return stockService.pageByProductSpu(dto);
    }

    /**
     * 库存明细
     */
    @PostMapping("/detail")
    public Stock detail(@RequestBody StockDto dto) {
        return stockService.detail(dto);
    }

    /**
     * 手动入库
     */
    @PostMapping("/add")
    public void add(@RequestBody StockDto stockDto) {
        stockService.add(stockDto);
    }

    /**
     * 手动出库
     */
    @PostMapping("/edit")
    public void edit(@RequestBody StockDto stockDto) {
        stockService.edit(stockDto);
    }

    /**
     * 库存删除
     */
    @PostMapping("/delete")
    public void delete(@RequestBody BaseSelectDto dto) {
        stockService.delete(dto.getId());
    }

    /**
     * 次品转良品
     *
     * @return
     */
    @PostMapping("/defectiveToQualified")
    public void defectiveToQualified(@RequestBody Stock stock) {
        stockService.defectiveToQualified(stock);
    }

    /**
     * 良品转次品
     *
     * @return
     */
    @PostMapping("/qualifiedToDefective")
    public void qualifiedToDefective(@RequestBody Stock stock) {
        stockService.qualifiedToDefective(stock);
    }

    /**
     * 获取库存产品部门列表
     */
    @GetMapping("/getDepts")
    public List<SysDept> getDepts() {
        return stockService.getDepts();
    }

    /**
     * 库存分析
     */
    @PostMapping("/stockAnalysis")
    public Page<StockAnalysis> stockAnalysis(@RequestBody StockSelectDto dto) {
        return stockService.stockAnalysis(dto);
    }

    /**
     * 库存分析导出excel
     */
    @PostMapping("/stockAnalysis/exportExcel")
    public void exportExcel(@RequestBody StockSelectDto dto, HttpServletResponse httpServletResponse) {
        stockService.exportExcel(dto, httpServletResponse);
    }

    /**
     * 出入库明细产品导入
     */
    @PostMapping("/excelImport")
    public List<IssueImportExcelBo> issueExcelImport(@RequestParam("file") MultipartFile file, Long warehouseId) {
        return stockService.issueExcelImport(file, warehouseId);
    }

}
