package com.fjhx.wms.service.stock.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.common.utils.Assert;
import com.fjhx.item.entity.product.po.ProductInfo;
import com.fjhx.item.service.product.ProductInfoService;
import com.fjhx.item.util.excel.util.ExcelUtil;
import com.fjhx.kd100.entity.company.po.CompanyInfo;
import com.fjhx.kd100.service.company.CompanyInfoService;
import com.fjhx.tenant.utils.DictUtils;
import com.fjhx.wms.entity.LogisticsInfosBo;
import com.fjhx.wms.entity.arrival.po.ArrivalStockRecords;
import com.fjhx.wms.entity.arrival.po.ArrivalStockRecordsDetails;
import com.fjhx.wms.entity.stock.bo.StockJournalDetailsExcelBo;
import com.fjhx.wms.entity.stock.dto.StockJournalDetailsDto;
import com.fjhx.wms.entity.stock.dto.StockJournalSelectDto;
import com.fjhx.wms.entity.stock.emums.JournalType;
import com.fjhx.wms.entity.stock.po.Stock;
import com.fjhx.wms.entity.stock.po.StockJournal;
import com.fjhx.wms.entity.stock.po.StockJournalDetails;
import com.fjhx.wms.entity.stock.po.StockJournalDetailsEditRecord;
import com.fjhx.wms.entity.stock.po.StockWait;
import com.fjhx.wms.entity.stock.po.StockWaitDetails;
import com.fjhx.wms.entity.stock.vo.StockJournalDetailsVo;
import com.fjhx.wms.entity.stock.vo.StockWaitVo;
import com.fjhx.wms.entity.warehouse.po.Warehouse;
import com.fjhx.wms.mapper.stock.StockJournalDetailsMapper;
import com.fjhx.wms.service.WmsService;
import com.fjhx.wms.service.arrival.ArrivalStockRecordsDetailsService;
import com.fjhx.wms.service.arrival.ArrivalStockRecordsService;
import com.fjhx.wms.service.stock.StockJournalDetailsEditRecordService;
import com.fjhx.wms.service.stock.StockJournalDetailsService;
import com.fjhx.wms.service.stock.StockJournalService;
import com.fjhx.wms.service.stock.StockService;
import com.fjhx.wms.service.stock.StockWaitDetailsService;
import com.fjhx.wms.service.stock.StockWaitService;
import com.fjhx.wms.service.warehouse.WarehouseService;
import com.ruoyi.common.core.domain.BasePo;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.system.utils.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.function.Function;
import java.util.stream.Collectors;


/**
 * <p>
 * 出入库记录明细 服务实现类
 * </p>
 *
 * @author
 * @since 2023-03-20
 */
@Service
public class StockJournalDetailsServiceImpl extends ServiceImpl<StockJournalDetailsMapper, StockJournalDetails> implements StockJournalDetailsService {
    @Autowired
    private ProductInfoService productInfoService;
    @Autowired
    private StockWaitService stockWaitService;
    @Autowired
    private StockService stockService;
    @Autowired
    private StockJournalService stockJournalService;
    @Autowired
    private StockJournalDetailsEditRecordService stockJournalDetailsEditRecordService;
    @Autowired
    private StockWaitDetailsService stockWaitDetailsService;
    @Autowired
    private ArrivalStockRecordsDetailsService arrivalStockRecordsDetailsService;
    @Autowired
    private ArrivalStockRecordsService arrivalStockRecordsService;
    @Autowired
    private StockJournalDetailsService stockJournalDetailsService;
    @Autowired
    private WarehouseService warehouseService;
    @Autowired
    private CompanyInfoService companyInfoService;
    @Autowired
    private WmsService wmsService;

    /**
     * 分页公共参数
     */
    private IWrapper<StockJournalDetails> getPageWrapper(StockJournalSelectDto dto) {

        IWrapper<StockJournalDetails> wrapper = getWrapper();
        wrapper.eq("sj.op_type", dto.getOpType());
        wrapper.eq("sj.type", dto.getType());
        wrapper.eq("sj.warehouse_id", dto.getWarehouseId());
        wrapper.eq("sj.to_warehouse_id", dto.getToWarehouseId());
        wrapper.eq("sjd.product_id", dto.getProductId());

        if (ObjectUtil.isNotEmpty(dto.getKeyword())) {
            //产品编号过滤
            List<Long> productIds = productInfoService.listObject(ProductInfo::getId, q -> q.like(ProductInfo::getCustomCode, dto.getKeyword()).or()
                    .like(ProductInfo::getName, dto.getKeyword()));
            //仓库信息过滤
            List<Long> warehouseIds = warehouseService.listObject(Warehouse::getId, q -> q.like(Warehouse::getName, dto.getKeyword()));

            wrapper.and(q -> q.in("sj.warehouse_id", warehouseIds).or()
                    .like("sj.link_code", dto.getKeyword()).or()
                    .in("sjd.product_id", productIds)
            );
        }

        //时间范围过滤
        wrapper.ge("sjd", StockJournalDetails::getCreateTime, dto.getBeginTime());
        wrapper.le("sjd", StockJournalDetails::getCreateTime, dto.getEndTime());

        //count方法手动赋值逻辑删除租户
        wrapper.eq("sj.del_flag", 0);
        wrapper.eq("sj.tenant_id", SecurityUtils.getTenantId());

        //排序
        wrapper.orderByDesc("sjd", StockJournalDetails::getCreateTime);

        return wrapper;
    }

    @Override
    public Page<StockJournalDetailsVo> getPage(StockJournalSelectDto dto) {
        IWrapper<StockJournalDetails> wrapper = getPageWrapper(dto);

        Page<Object> page1 = dto.getPage();
        page1.setCountId("getPageCount");

        Page<StockJournalDetailsVo> page = this.baseMapper.getPage(page1, wrapper);
        List<StockJournalDetailsVo> records = page.getRecords();
        if (ObjectUtil.isEmpty(records)) {
            return page;
        }
        // 赋值产品属性
        productInfoService.attributeAssign(records, StockJournalDetailsVo::getProductId, (item, product) -> {
            item.setProductCode(product.getCode());
            item.setProductUnit(product.getUnit());
            item.setProductType(product.getType());
            item.setProductName(product.getName());
            item.setProductSpec(product.getSpec());
            item.setProductCustomCode(product.getCustomCode());
        });
        //赋值操作人名称
        UserUtil.assignmentNickName(records, StockJournalDetailsVo::getCreateUser, StockJournalDetailsVo::setOpUserName);

        //赋值仓库名称
        warehouseService.attributeAssign(records, StockJournalDetailsVo::getWarehouseId, (item, warehouse) -> {
            item.setWarehouseName(warehouse.getName());
        });

        //赋值发货信息
        stockWaitService.attributeAssign(records, StockJournalDetailsVo::getBusinessId, (item, stockWait) -> {
            String victoriatouristJson = stockWait.getVictoriatouristJson();
            if (ObjectUtil.isNotEmpty(victoriatouristJson)) {
                JSONObject jsonObject = JSONObject.parseObject(victoriatouristJson);
                item.setDeliverGoodsId(jsonObject.getLong("deliverGoodsId"));
            }
        });

        //赋值物流单号信息
        List<Long> logisticsBusinessIds = new ArrayList<>();
        logisticsBusinessIds.addAll(records.stream().map(StockJournalDetailsVo::getBusinessId).distinct().filter(Objects::nonNull).collect(Collectors.toList()));
        logisticsBusinessIds.addAll(records.stream().map(StockJournalDetailsVo::getDeliverGoodsId).distinct().filter(Objects::nonNull).collect(Collectors.toList()));
        if (ObjectUtil.isNotEmpty(logisticsBusinessIds)) {
            List<LogisticsInfosBo> logisticsInfosByBusinessIds = wmsService.getLogisticsInfosByBusinessIds(logisticsBusinessIds);
            Map<Long, LogisticsInfosBo> collect = logisticsInfosByBusinessIds.stream().collect(Collectors.toMap(LogisticsInfosBo::getBusinessId, Function.identity()));
            for (StockJournalDetailsVo record : records) {
                LogisticsInfosBo orDefault = collect.getOrDefault(record.getBusinessId(), collect.get(record.getDeliverGoodsId()));
                if (ObjectUtil.isNotEmpty(orDefault)) {
                    record.setLogisticsCompanyCode(orDefault.getLogisticsCompanyCode());
                    record.setLogisticsCode(orDefault.getCode());
                }
            }
        }

        //赋值物流公司名称
        List<String> companyCods = records.stream().map(StockJournalDetailsVo::getLogisticsCompanyCode).distinct().filter(item -> ObjectUtil.isNotEmpty(item)).collect(Collectors.toList());
        if (ObjectUtil.isNotEmpty(companyCods)) {
            Map<String, String> companyMap = companyInfoService.mapKV(CompanyInfo::getCode, CompanyInfo::getName, q -> q.in(CompanyInfo::getCode, companyCods));
            for (StockJournalDetailsVo record : records) {
                record.setLogisticsCompanyName(companyMap.get(record.getLogisticsCompanyCode()));
            }
        }

        return page;
    }

    @Override
    public Page<StockJournalDetailsVo> getPageByWdly(StockJournalSelectDto dto) {
        Page<StockJournalDetailsVo> page = getPage(dto);
        List<StockJournalDetailsVo> records = page.getRecords();
        if (ObjectUtil.isEmpty(records)) {
            return page;
        }

        List<Long> swBusinessIds = records.stream().map(StockJournalDetailsVo::getBusinessId).distinct().filter(Objects::nonNull).collect(Collectors.toList());

        if (ObjectUtil.isNotEmpty(swBusinessIds)) {
            //给 采购退货 采购到货 的数据赋值关联单号
            Map<Long, StockWait> stockWaitMap = stockWaitService.mapKEntity(StockWait::getId, q -> q.in(StockWait::getId, swBusinessIds));

            //给 到货质检 赋值关联单号
            IWrapper<StockWait> wrapper = IWrapper.getWrapper();
            wrapper.in("qi", StockWait::getId, swBusinessIds);
            List<StockWaitVo> stockWaitInfo = baseMapper.getStockWaitInfo(wrapper);
            Map<Long, String> stockWaitInfoMap = stockWaitInfo.stream().collect(Collectors.toMap(StockWait::getId, StockWait::getBusinessCode));

            for (StockJournalDetailsVo record : records) {
                //给 采购退货 采购到货 的数据赋值关联单号
                StockWait stockWait = stockWaitMap.get(record.getBusinessId());
                if (ObjectUtil.isNotEmpty(stockWait)) {
                    record.setLinkCode(stockWait.getBusinessCode());
                    continue;
                }
                //给 到货质检 赋值关联单号
                String linkCode = stockWaitInfoMap.get(record.getBusinessId());
                if (ObjectUtil.isNotEmpty(linkCode)) {
                    record.setLinkCode(linkCode);
                }
            }
        }
        return page;
    }

    @Override
    public StockJournalDetailsVo detail(Long id) {
        StockJournalDetails StockJournalDetails = this.getById(id);
        StockJournalDetailsVo result = BeanUtil.toBean(StockJournalDetails, StockJournalDetailsVo.class);
        return result;
    }

    @Override
    public void add(StockJournalDetailsDto stockJournalDetailsDto) {
        this.save(stockJournalDetailsDto);
    }

    @Override
    public void edit(StockJournalDetailsDto stockJournalDetailsDto) {
        this.updateById(stockJournalDetailsDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

    /**
     * 维多利亚流水数量修正
     */
    @DSTransactional
    @Override
    public synchronized void editQuantity(StockJournalDetails dto) {
        Assert.notEmpty(dto.getId(), "流水明细id不能为空");
        StockJournalDetails stockJournalDetails = getById(dto.getId());

        //根据流水获取入库批次
        List<Long> stockJournalDetailsList = stockJournalDetailsService.listObject(StockJournalDetails::getId, q -> q
                .eq(StockJournalDetails::getBusinessDetailsId, stockJournalDetails.getBusinessDetailsId())
                .eq(StockJournalDetails::getProductId, stockJournalDetails.getProductId())
                .eq(StockJournalDetails::getQuantity, stockJournalDetails.getQuantity())
                .orderByAsc(StockJournalDetails::getCreateTime)
        );
        int batchIndex = stockJournalDetailsList.indexOf(stockJournalDetails.getId());
        if (batchIndex == -1) {
            throw new ServiceException("找不到入库批次信息");
        }

        //保存操作记录
        StockJournalDetailsEditRecord stockJournalDetailsEditRecord = new StockJournalDetailsEditRecord();
        stockJournalDetailsEditRecord.setStockJournalDetailsId(dto.getId());
        stockJournalDetailsEditRecord.setOriginalQuantity(stockJournalDetails.getQuantity());
        stockJournalDetailsEditRecord.setChangeQuantity(dto.getQuantity());

        stockJournalDetailsEditRecordService.save(stockJournalDetailsEditRecord);

        //修改库存
        StockJournal stockJournal = stockJournalService.getById(stockJournalDetails.getStockJournalId());
        BigDecimal subtract;
        if (stockJournal.getOpType() == 1) {
            subtract = dto.getQuantity().subtract(stockJournalDetails.getQuantity());
        } else if (stockJournal.getOpType() == 2) {
            subtract = stockJournalDetails.getQuantity().subtract(dto.getQuantity());
        } else {
            throw new ServiceException("未知流水操作类型");
        }
        Stock changeStock = new Stock();
        changeStock.setProductId(stockJournalDetails.getProductId());
        changeStock.setQuantity(subtract);
        stockService.ModifyInventory(null, 3, Collections.singletonList(changeStock), stockJournal.getWarehouseId());

        //修改待出入库数据
        Long stockWaitDetailsId = stockJournalDetails.getBusinessDetailsId();
        Assert.notEmpty(stockWaitDetailsId, "无法关联入库信息，操作失败");


        boolean update1 = stockWaitDetailsService.update(q -> q
                .eq(StockWaitDetails::getId, stockWaitDetailsId)
                .setSql("receipt_quantity = receipt_quantity+" + subtract)
        );
        if (!update1) {
            throw new ServiceException("操作失败请重试");
        }
        //修改待入库状态
        StockWaitDetails stockWaitDetails = stockWaitDetailsService.getById(stockWaitDetailsId);
        Assert.notEmpty(stockWaitDetails, "无法查询到关联待入库信息，操作失败");
        StockWait stockWait = stockWaitService.getById(stockWaitDetails.getStockWaitId());
        if (stockWait.getStatus() != 3) {
            //修改状态
            stockWaitDetailsService.changeStockWaitStatus(stockWait.getId());
        }

        //修改待质检数量
        List<Long> arrivalStockRecordsDetailsIds = arrivalStockRecordsDetailsService.listObject(ArrivalStockRecordsDetails::getId, q -> q
                .eq(ArrivalStockRecordsDetails::getBusinessDetailsId, stockJournalDetails.getBusinessDetailsId())
                .eq(ArrivalStockRecordsDetails::getProductInfoId, stockJournalDetails.getProductId())
                .eq(ArrivalStockRecordsDetails::getQuantity, stockJournalDetails.getQuantity())
                .orderByAsc(ArrivalStockRecordsDetails::getCreateTime)
        );

        //根据批次获取到入库记录
        Long arrivalStockRecordsDetailsId = arrivalStockRecordsDetailsIds.get(batchIndex);

        ArrivalStockRecordsDetails oldarrivalStockRecordsDetails = arrivalStockRecordsDetailsService.getById(arrivalStockRecordsDetailsId);
        Assert.notEmpty(oldarrivalStockRecordsDetails, "查询不到相关质检信息");
        if (oldarrivalStockRecordsDetails.getQualityQuantity().compareTo(oldarrivalStockRecordsDetails.getQuantity()) >= 0) {
            throw new ServiceException("该记录已质检完成禁止修改");
        }

        boolean update2 = arrivalStockRecordsDetailsService.update(q -> q
                .eq(ArrivalStockRecordsDetails::getId, arrivalStockRecordsDetailsId)
                .setSql("quantity = quantity+" + subtract)
        );
        if (!update2) {
            throw new ServiceException("操作失败请重试");
        }
        //更新质检记录总数量
        wmsService.updateQualityTotalQuantityByArrivalRecordDetailsId(arrivalStockRecordsDetailsId, subtract);

        //修改质检状态
        ArrivalStockRecordsDetails arrivalStockRecordsDetails = arrivalStockRecordsDetailsService.getOne(q -> q
                .eq(ArrivalStockRecordsDetails::getBusinessDetailsId, stockWaitDetailsId)
        );
        updateArrivalStockRecordsStatus(arrivalStockRecordsDetails.getArrivalStockRecordsId());


        stockJournalDetails.setQuantity(dto.getQuantity());
        this.updateById(stockJournalDetails);
    }

    private void updateArrivalStockRecordsStatus(Long arrivalStockRecordsId) {
        //更新待出库记录状态
        int statusFlag = 0;
        List<ArrivalStockRecordsDetails> arrivalStockRecordsDetails = arrivalStockRecordsDetailsService.list(q -> q.eq(ArrivalStockRecordsDetails::getArrivalStockRecordsId, arrivalStockRecordsId));
        for (ArrivalStockRecordsDetails stockRecordsDetails : arrivalStockRecordsDetails) {
            //计算已经完全质检的数量
            BigDecimal qualityQuantity1 = ObjectUtil.isEmpty(stockRecordsDetails.getQualityQuantity()) ? BigDecimal.ZERO : stockRecordsDetails.getQualityQuantity();
            if (qualityQuantity1.compareTo(stockRecordsDetails.getQuantity()) >= 0) {
                statusFlag++;
            }
        }
        Integer qualityStatus;
        if (statusFlag == arrivalStockRecordsDetails.size()) {
            qualityStatus = 2;//质检完成
        } else {
            qualityStatus = 1;//部分质检
        }
        arrivalStockRecordsService.update(q -> q
                .eq(ArrivalStockRecords::getId, arrivalStockRecordsId)
                .set(ArrivalStockRecords::getQualityStatus, qualityStatus)
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
                .set(BasePo::getUpdateTime, new Date())
        );
    }


    /**
     * 出入库流水Excel导出
     */
    @Override
    public void exportExcel(StockJournalSelectDto dto, HttpServletResponse httpServletResponse) {
        IWrapper<StockJournalDetails> wrapper = getPageWrapper(dto);

        //产品单位字典读取
        Map<String, String> unitDictMap = DictUtils.getDictMap("unit");

        List<StockJournalDetailsVo> list = baseMapper.getList(wrapper);
        List<StockJournalDetailsExcelBo> stockJournalDetailsExcelBos = BeanUtil.copyToList(list, StockJournalDetailsExcelBo.class);
        for (StockJournalDetailsExcelBo stockJournalDetailsExcelBo : stockJournalDetailsExcelBos) {

            //赋值产品单位字典
            stockJournalDetailsExcelBo.setProductUnit(unitDictMap.get(stockJournalDetailsExcelBo.getProductUnit()));

            //赋值操作类型
            stockJournalDetailsExcelBo.setOpTypeName(Objects.equals(stockJournalDetailsExcelBo.getOpType(), 1) ? "入库" : "出库");
            //赋值业务类型
            stockJournalDetailsExcelBo.setTypeName(JournalType.getJournalTypeVal(stockJournalDetailsExcelBo.getType()));
        }

        //导出Excel
        ExcelUtil.export(httpServletResponse, stockJournalDetailsExcelBos, StockJournalDetailsExcelBo.class);
    }

}
